from typing import Any

from fastapi import HTTPException, status
from .user_auth_schemas import UserAuthResponse
from test_infinity_stones.network_router import HttpRequest, APIAuth, HttpMethod, NetworkRouter
from test_infinity_stones.config import Settings
import logging

logger = logging.getLogger()

class Authenticator:
    def __init__(self, settings: Settings, network_router: NetworkRouter = NetworkRouter()):
        self._settings = settings
        self._network_router = network_router


    async def authenticate_user(self, request_headers: dict[str, str]) -> UserAuthResponse:
        authorization = request_headers.get("Authorization")

        if not authorization:
            raise HTTPException(status_code=401)

        split_auth_header = authorization.split(" ")

        if len(split_auth_header) < 2:
            raise HTTPException(status_code=401)

        access_token = split_auth_header[1]

        request = HttpRequest[Any](
            method=HttpMethod.GET,
            base_url=self._settings.AUTH_SERVICE_BASE_URL,
            path=self._settings.AUTHENTICATE_USER_ENDPOINT_PATH,
            headers=APIAuth.bearer(access_token),
        )

        try:
            response = await self._network_router.execute_http_request(request)

            if response.status_code != 200:
                raise HTTPException(status_code=response.status_code, detail=response.body)

        except Exception as e:
            logger.error(f"Error while authenticating user {e}")
            raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED)

        return UserAuthResponse(**response.body)